VERSION 5.00
Begin VB.Form frmMain 
   BorderStyle     =   3  'Fixed Dialog
   Caption         =   "DirectX-Graphics: Lesson 01"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   4680
   ControlBox      =   0   'False
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   213
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   312
   ShowInTaskbar   =   0   'False
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'###############################
'
'           Title: DirectX-Graphics Lesson 08
'           Desc: Loading Externally created objects into D3D8
'           Written by: Jack Hoxley
'           Date: December 21st 2000
'           Contact: Jollyjeffers@Greenonions.netscapeonline.co.uk
'
'###############################

Dim DX As DirectX8 'The master Object, everything comes from here
Dim D3D As Direct3D8 'This controls all things 3D
Dim D3DDevice As Direct3DDevice8 'This actually represents the hardware doing the rendering
Dim D3DX As D3DX8 '//A helper library
Dim VBuffer As Direct3DVertexBuffer8 '//Stores our Geometry....

Dim bRunning As Boolean 'Controls whether the program is running or not...

'//two mathematical constants that are useful for 3D graphics...
Const pi As Single = 3.14159265358979   'can be worked out using (4*atn(1))
Const Rad = pi / 180 'Multiply degrees by this much to get result in radians

'#########
'## UDTs ##
'########
Private Enum VisTest
    PARTLYVISIBLE = 0
    TOTALLYVISIBLE = 1
    TOTALLYINVISIBLE = 2
    TOSCREENLEFT = 3
    TOSCREENRIGHT = 4
    TOSCREENTOP = 5
    TOSCREENBOTTOM = 6
    TOSCREENBACK = 7
End Enum

Private Type LVertex
    P As D3DVECTOR
    Color As Long
End Type

Const FVF_LVertex = (D3DFVF_XYZ Or D3DFVF_DIFFUSE)

Private Type Box3D
    Coord(0 To 7) As D3DVECTOR '//makes things easier storing them as vectors
End Type

Private FrustumPlanes(0 To 5) As D3DPLANE

'############
'## VERTICES ##
'###########

Private VertList(0 To 3) As LVertex 'a simple tile...

'#########
'## FONTS ##
'#########
Dim MainFont As D3DXFont
Dim MainFontDesc As IFont
Dim TextRect As RECT
Dim fnt As New StdFont

'############
'##  MATRICES ##
'############
Dim matWorld As D3DMATRIX '//How the vertices are positioned
Dim matView As D3DMATRIX '//Where the camera is/where it's looking
Dim matProj As D3DMATRIX '//How the camera projects the 3D world onto the 2D screen

'########################
'## FRAME RATE CALCULATIONS ##
'#######################
Private Declare Function GetTickCount Lib "kernel32" () As Long
Dim FPS_LastCheck As Long
Dim FPS_Count As Long
Dim FPS_Current As Integer

'################
'## GENERAL STUFF ##
'###############

Dim DevInfo As String
Dim TileCurrPos As D3DVECTOR
Dim TileCurrScale As Single




'// Initialise : This procedure kick starts the whole process.
'// It'll return true for success, false if there was an error.
Public Function Initialise() As Boolean
On Error GoTo ErrHandler:

Dim DispMode As D3DDISPLAYMODE '//Describes our Display Mode
Dim D3DWindow As D3DPRESENT_PARAMETERS '//Describes our Viewport

Set DX = New DirectX8  '//Create our Master Object
Set D3D = DX.Direct3DCreate() '//Make our Master Object create the Direct3D Interface
Set D3DX = New D3DX8 '//Create our helper library...

'//We're going to use Fullscreen mode because I prefer it to windowed mode :)

'DispMode.Format = D3DFMT_X8R8G8B8
DispMode.Format = D3DFMT_R5G6B5 'If this mode doesn't work try the commented one above...
DispMode.Width = 640
DispMode.Height = 480

D3DWindow.SwapEffect = D3DSWAPEFFECT_FLIP
D3DWindow.BackBufferCount = 1 '//1 backbuffer only
D3DWindow.BackBufferFormat = DispMode.Format 'What we specified earlier
D3DWindow.BackBufferWidth = DispMode.Width
D3DWindow.BackBufferHeight = DispMode.Height
D3DWindow.hDeviceWindow = frmMain.hWnd
D3DWindow.EnableAutoDepthStencil = 1

If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D16) = D3D_OK Then
    'We can use a 16 bit Z-Buffer
    D3DWindow.AutoDepthStencilFormat = D3DFMT_D16 '//16 bit Z-Buffer
Else
    'We could now check for different modes available...
End If

Dim DevCaps As D3DCAPS8
D3D.GetDeviceCaps D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DevCaps

'//For Pure Device processing:
If (DevCaps.DevCaps And D3DDEVCAPS_PUREDEVICE) Then
    '//CREATE A PURE DEVICE
    Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_PUREDEVICE, _
                                                        D3DWindow)
    DevInfo = "Pure Hardware Device / " & DispMode.Width & "x" & DispMode.Height
Else
    '//CHECK FOR HW TnL
        '//For Hardware vertex processing:
        If (DevCaps.DevCaps And D3DDEVCAPS_HWTRANSFORMANDLIGHT) Then
            '//CREATE A HW TnL DEVICE
            Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, _
                                                        D3DWindow)
            DevInfo = "Hardware TnL Device / " & DispMode.Width & "x" & DispMode.Height
        Else
            '//USE SOFTWARE DEVICE
            Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, _
                                                        D3DWindow)
            DevInfo = "SW VP / HAL Device / " & DispMode.Width & "x" & DispMode.Height
        End If
End If

DevInfo = DevInfo & " 16 bit"

'//Set the vertex shader to use our vertex format
D3DDevice.SetVertexShader FVF_LVertex

'//lit vertices dont need lighting
'   so we disable it...
D3DDevice.SetRenderState D3DRS_LIGHTING, 0

'//We need to enable our Z Buffer
D3DDevice.SetRenderState D3DRS_ZENABLE, 1

D3DDevice.SetTextureStageState 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR
D3DDevice.SetTextureStageState 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR

'############
'##  MATRICES ##
'############

'//1. The World Matrix
D3DXMatrixIdentity matWorld
D3DDevice.SetTransform D3DTS_WORLD, matWorld 'commit this matrix to the device

'//2. The View Matrix
D3DXMatrixLookAtLH matView, MakeVector(0, 15, 20), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
D3DDevice.SetTransform D3DTS_VIEW, matView

'//3. The projection Matrix
D3DXMatrixPerspectiveFovLH matProj, pi / 3, 1, 0.1, 75
D3DDevice.SetTransform D3DTS_PROJECTION, matProj

'##########
'## FONTS ##
'#########
fnt.Name = "Verdana"
fnt.Size = 12
Set MainFontDesc = fnt
Set MainFont = D3DX.CreateFont(D3DDevice, MainFontDesc.hFont)

'############
'## TEXTURES ##
'###########

If InitialiseGeometry() = False Then GoTo ErrHandler: 'We must create the geometry - otherwise there aint no point in going on

Initialise = True

Exit Function
ErrHandler:
'//We failed; for now we wont worry about why.
Debug.Print "Error Number Returned: " & Err.Number
Initialise = False
End Function

Public Sub Render()
'//0. Any variables required
    Dim I As Long 'Loop variable
    Dim vec3D As D3DVECTOR, vec2D As D3DVECTOR, VP As D3DVIEWPORT8
    Dim res As VisTest, Str As String, box As Box3D, TileDrawn As Boolean
    
'//1. We need to clear the render device before we can draw anything
'       This must always happen before you start rendering stuff...
D3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET Or D3DCLEAR_ZBUFFER, &H8080FF, 1#, 0  '//Clear the screen black

'//2. Rendering the graphics...

D3DDevice.BeginScene
    'All rendering calls go between these two lines
    
    '//Draw the geometry
    
    box.Coord(0) = VertList(0).P
    box.Coord(1) = VertList(1).P
    box.Coord(2) = VertList(2).P
    box.Coord(3) = VertList(3).P
    box.Coord(4) = VertList(0).P 'because it's a flat tile, we make the cube flat...
    box.Coord(5) = VertList(1).P
    box.Coord(6) = VertList(2).P
    box.Coord(7) = VertList(3).P
    
    res = ProjectionVisibilityBoundingBox(box)
    
    If res = PARTLYVISIBLE Or res = TOTALLYVISIBLE Then
        TileDrawn = True
        D3DDevice.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, VertList(0), Len(VertList(0))
    End If
    
    '//Draw the frame rate
    TextRect.Top = 0
    TextRect.bottom = 20
    TextRect.Right = 75
    D3DX.DrawText MainFont, &HFFFFCC00, CStr(FPS_Current) & "fps", TextRect, DT_TOP Or DT_LEFT

    

    
    TextRect.Top = 20
    TextRect.bottom = 40
    TextRect.Right = 350
    D3DX.DrawText MainFont, &HFFFFCC00, DevInfo, TextRect, DT_TOP Or DT_LEFT
    

    TextRect.Top = 40
    TextRect.bottom = 60
    TextRect.Left = 0
    TextRect.Right = 450
    res = ProjectionVisibilityTesting(VertList(0).P)
    Select Case res
        Case 0
            Str = "Vertex 0 is partly visible"
        Case Is > 2
            Str = "Vertex 0 is totally invisible"
        Case 1
            Str = "Vertex 0 is totally visible"
    End Select
    
    D3DX.DrawText MainFont, &HFFFFCC00, Str, TextRect, DT_TOP Or DT_LEFT
    
    TextRect.Top = 60
    TextRect.bottom = 80
    TextRect.Right = 350
    D3DX.DrawText MainFont, &HFFFFCC00, "Bounding Box: Tile was drawn = " & CStr(TileDrawn), TextRect, DT_TOP Or DT_LEFT

    
        
D3DDevice.EndScene

'//3. Update the frame to the screen...
'       This is the same as the Primary.Flip method as used in DirectX 7
'       These values below should work for almost all cases...
D3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
End Sub

Private Sub Form_Click()
bRunning = False
End Sub


Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)
Select Case KeyCode
    Case vbKeyLeft
        TileCurrPos.X = TileCurrPos.X + 0.5
    Case vbKeyRight
        TileCurrPos.X = TileCurrPos.X - 0.5
    Case vbKeyUp
        TileCurrPos.Z = TileCurrPos.Z - 0.5
    Case vbKeyDown
        TileCurrPos.Z = TileCurrPos.Z + 0.5
    Case vbKeyAdd
        TileCurrScale = TileCurrScale + 0.25
    Case vbKeySubtract
        TileCurrScale = TileCurrScale - 0.25
    Case vbKeyR
        TileCurrScale = 1
        TileCurrPos = MakeVector(0, 0, 0)
End Select
End Sub

Private Sub Form_Load()
Dim RotateAngle As Single
Dim matTemp As D3DMATRIX '//To hold temporary stuff...

Me.Show '//Make sure our window is visible

bRunning = Initialise()
Debug.Print "Device Creation Return Code : ", bRunning 'So you can see what happens...

Do While bRunning
    'We're going to update our world matrix so that the object does
    'something vaguely interesting... :)
    
    RotateAngle = RotateAngle + 0.3 'alter this if it appears to go stupidly fast... who need artificial motion blur... hehe
    If RotateAngle >= 360 Then RotateAngle = RotateAngle - 360
    
    D3DXMatrixIdentity matWorld '//Reset our world matrix
       
    D3DXMatrixIdentity matTemp
    D3DXMatrixRotationY matTemp, RotateAngle * (pi / 180)
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixScaling matTemp, TileCurrScale, TileCurrScale, TileCurrScale
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixTranslation matTemp, TileCurrPos.X, TileCurrPos.Y, TileCurrPos.Z
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
       
    D3DDevice.SetTransform D3DTS_WORLD, matWorld
        
    Render '//Update the frame...
    
    '//Frame rate calculations
    If GetTickCount() - FPS_LastCheck >= 1000 Then
        FPS_Current = FPS_Count
        FPS_Count = 0 'reset the counter
        FPS_LastCheck = GetTickCount()
    End If
    FPS_Count = FPS_Count + 1
    
    DoEvents '//Allow windows time to think; otherwise you'll get into a really tight (and bad) loop...
    
Loop '//Begin the next frame...

'//If we've gotten to this point the loop must have been terminated
'   So we need to clean up after ourselves. This isn't essential, but it's
'   good coding practise.

On Error Resume Next 'If the objects were never created;
'                               (the initialisation failed) we might get an
'                               error when freeing them... which we need to
'                               handle, but as we're closing anyway...
Set D3DDevice = Nothing
Set D3D = Nothing
Set DX = Nothing
Debug.Print "All Objects Destroyed"

'//Final termination:
Unload Me
End
End Sub





Private Function InitialiseGeometry() As Boolean
    
    
On Error GoTo BailOut: '//Setup our Error handler

VertList(2).P = MakeVector(-1, 0, -1)
VertList(3).P = MakeVector(1, 0, -1)
VertList(0).P = MakeVector(-1, 0, 1)
VertList(1).P = MakeVector(1, 0, 1)

'//NB: remember, RGB() must be in BGR order...
VertList(0).Color = RGB(0, 0, 255)
VertList(1).Color = RGB(0, 255, 0)
VertList(2).Color = RGB(255, 0, 0)
VertList(3).Color = RGB(0, 125, 255)

InitialiseGeometry = True
Exit Function
BailOut:
Debug.Print "Error occured in InitGeometry() Code"
InitialiseGeometry = False
End Function


'## This function takes a 3D point and transforms it into screenspace,
'## it also checks against the depth buffer and screen boundaries, resulting
'## in either Totally visible or Partly visible return values.
Private Function ProjectionVisibilityTesting(Point3D As D3DVECTOR) As VisTest
'//0. Any variables
    Dim vRet As D3DVECTOR
    Dim VP As D3DVIEWPORT8
    
'//1. Collect the data
    D3DDevice.GetViewport VP
    D3DXVec3Project vRet, Point3D, VP, matProj, matView, matWorld
    
'//2. correlate data
    If vRet.X < VP.X Then
        ProjectionVisibilityTesting = TOSCREENLEFT
        Exit Function
        
    ElseIf vRet.X > VP.Width Then
        ProjectionVisibilityTesting = TOSCREENRIGHT
        Exit Function
        
    ElseIf vRet.Y < VP.Y Then
        ProjectionVisibilityTesting = TOSCREENTOP
        Exit Function
        
    ElseIf vRet.Y > VP.Height Then
        ProjectionVisibilityTesting = TOSCREENBOTTOM
        Exit Function
        
    ElseIf vRet.Z > VP.MaxZ Then
        ProjectionVisibilityTesting = TOSCREENBACK
        Exit Function
    
    ElseIf vRet.Z < VP.MinZ Then
        ProjectionVisibilityTesting = TOSCREENBOTTOM
        Exit Function
        
    Else
        ProjectionVisibilityTesting = TOTALLYVISIBLE
        Exit Function
        
    End If
    
End Function

Private Function ProjectionVisibilityBoundingBox(BBox As Box3D) As VisTest
'//0. Any variables
    Dim Results(0 To 7) As VisTest
    
'//1. Perform tests
    Results(0) = ProjectionVisibilityTesting(BBox.Coord(0))
    Results(1) = ProjectionVisibilityTesting(BBox.Coord(1))
    Results(2) = ProjectionVisibilityTesting(BBox.Coord(2))
    Results(3) = ProjectionVisibilityTesting(BBox.Coord(3))
    Results(4) = ProjectionVisibilityTesting(BBox.Coord(4))
    Results(5) = ProjectionVisibilityTesting(BBox.Coord(5))
    Results(6) = ProjectionVisibilityTesting(BBox.Coord(6))
    Results(7) = ProjectionVisibilityTesting(BBox.Coord(7))
    
'//2. correlate the data
    
    '//handle simplest cases first:
    If Results(0) = TOTALLYVISIBLE And Results(1) = TOTALLYVISIBLE And Results(2) = TOTALLYVISIBLE And Results(3) = TOTALLYVISIBLE And Results(4) = TOTALLYVISIBLE And Results(5) = TOTALLYVISIBLE And Results(6) = TOTALLYVISIBLE And Results(7) = TOTALLYVISIBLE Then
        'if all of them are visible then return totally visible
        ProjectionVisibilityBoundingBox = TOTALLYVISIBLE
        Exit Function
        
    ElseIf Results(0) = TOTALLYVISIBLE Or Results(1) = TOTALLYVISIBLE Or Results(2) = TOTALLYVISIBLE Or Results(3) = TOTALLYVISIBLE Or Results(4) = TOTALLYVISIBLE Or Results(5) = TOTALLYVISIBLE Or Results(6) = TOTALLYVISIBLE Or Results(7) = TOTALLYVISIBLE Then
        'if any of them are in the viewing area, return partially visible
        ProjectionVisibilityBoundingBox = PARTLYVISIBLE
        Exit Function
    End If
    
    
    '//Handle the extremes/invisibility cases
    If Results(0) = TOSCREENBACK And Results(1) = TOSCREENBACK And Results(2) = TOSCREENBACK And Results(3) = TOSCREENBACK And Results(4) = TOSCREENBACK And Results(5) = TOSCREENBACK And Results(6) = TOSCREENBACK And Results(7) = TOSCREENBACK Then
        'all of the points are beyond the back clipping plain
        ProjectionVisibilityBoundingBox = TOTALLYINVISIBLE
        Exit Function
        
    ElseIf Results(0) = TOSCREENLEFT And Results(1) = TOSCREENLEFT And Results(2) = TOSCREENLEFT And Results(3) = TOSCREENLEFT And Results(4) = TOSCREENLEFT And Results(5) = TOSCREENLEFT And Results(6) = TOSCREENLEFT And Results(7) = TOSCREENLEFT Then
        'all of the points are to the left of the screen
        ProjectionVisibilityBoundingBox = TOTALLYINVISIBLE
        Exit Function
        
    ElseIf Results(0) = TOSCREENRIGHT And Results(1) = TOSCREENRIGHT And Results(2) = TOSCREENRIGHT And Results(3) = TOSCREENRIGHT And Results(4) = TOSCREENRIGHT And Results(5) = TOSCREENRIGHT And Results(6) = TOSCREENRIGHT And Results(7) = TOSCREENRIGHT Then
        'all of the points are to the right of the screen
        ProjectionVisibilityBoundingBox = TOTALLYINVISIBLE
        Exit Function
        
    ElseIf Results(0) = TOSCREENTOP And Results(1) = TOSCREENTOP And Results(2) = TOSCREENTOP And Results(3) = TOSCREENTOP And Results(4) = TOSCREENTOP And Results(5) = TOSCREENTOP And Results(6) = TOSCREENTOP And Results(7) = TOSCREENTOP Then
        'all of the points are off the top of the screen
        ProjectionVisibilityBoundingBox = TOTALLYINVISIBLE
        Exit Function
        
    ElseIf Results(0) = TOSCREENBOTTOM And Results(1) = TOSCREENBOTTOM And Results(2) = TOSCREENBOTTOM And Results(3) = TOSCREENBOTTOM And Results(4) = TOSCREENBOTTOM And Results(5) = TOSCREENBOTTOM And Results(6) = TOSCREENBOTTOM And Results(7) = TOSCREENBOTTOM Then
        'all of the points are off the bottom of the screen...
        ProjectionVisibilityBoundingBox = TOTALLYINVISIBLE
        Exit Function
        
    End If
    
    '//Handle the tricky cases, where they occupy two extremes...
        'these have already been handled by the above logic system, currently it will only
        'flag invisibility if the tile has ALL vertices in the same extreme, therefore allowing those
        'boxes that occupy two extremes. It also handles the sub-case of this where it's in two extremes
        'AND out of range - this is also handled, in order to have this case the vertices must be beyond 2 extremes,
        'and it's already handled for 1 extreme - so there's no need to check for two.
    
End Function

'//Borrowed from the SDK's D3DUTIL module... Computes the clip planes for the view frustum...
Sub ComputeClipPlanes(veye As D3DVECTOR, vat As D3DVECTOR, vUp As D3DVECTOR, fov As Single, front As Single, back As Single, aspect As Single)
    
    Dim vDir As D3DVECTOR
    Dim vright As D3DVECTOR
        
    Dim vFrontCenter As D3DVECTOR
    Dim vFrontUp As D3DVECTOR
    Dim vFrontRight As D3DVECTOR
    
    Dim vBackCenter As D3DVECTOR
    
    Dim vBackRight As D3DVECTOR
    Dim vbackLeft As D3DVECTOR
    
    Dim vBackRightTop As D3DVECTOR
    Dim vBackLeftTop As D3DVECTOR
    
    Dim vBackRightBot As D3DVECTOR
    Dim vBackLeftBot As D3DVECTOR
        
    Dim DX As Single
    Dim dY As Single
    
    'Establish our basis vector
    D3DXVec3Subtract vDir, vat, veye
    D3DXVec3Normalize vDir, vDir
    D3DXVec3Normalize vUp, vUp
    D3DXVec3Cross vright, vDir, vUp
    
    DX = Tan(fov / 2) * back
    dY = DX * aspect
        
    '
    '
    '              /|  vbackleft (top,bot)
    '             / |
    '        vfront |
    '           /|  |
    '       eye ----|  vbackcenter
    '           \|  |
    '            \  |dx
    '             \ |
    '              \|  vbackright (top,bot)
    '
    '
    
    
    'compute vbackcenter
    D3DXVec3Scale vBackCenter, vDir, back
    D3DXVec3Add vBackCenter, vBackCenter, veye
    
       
    'compute vbackright
    D3DXVec3Scale vBackRight, vright, DX
    D3DXVec3Add vBackRight, vBackCenter, vBackRight
    
    
    'compute vbackleft
    D3DXVec3Scale vbackLeft, vright, -DX
    D3DXVec3Add vbackLeft, vBackCenter, vbackLeft
 
    'compute vbackrighttop
    D3DXVec3Scale vBackRightTop, vUp, dY
    D3DXVec3Add vBackRightTop, vBackRight, vBackRightTop
    
    
    'compute vbacklefttop
    D3DXVec3Scale vBackLeftTop, vUp, dY
    D3DXVec3Add vBackLeftTop, vBackRight, vBackLeftTop
        
     'compute vbackrightbot
    D3DXVec3Scale vBackRightBot, vUp, -dY
    D3DXVec3Add vBackRightBot, vBackRight, vBackRightBot
       
    'compute vbackleftbot
    D3DXVec3Scale vBackLeftBot, vUp, -dY
    D3DXVec3Add vBackLeftBot, vBackRight, vBackLeftBot
    
        
    'compute vfrontcenter
    D3DXVec3Scale vFrontCenter, vDir, front
    D3DXVec3Add vFrontCenter, vFrontCenter, veye
 
    'compute vfrontright
    D3DXVec3Scale vFrontRight, vright, DX
    D3DXVec3Add vFrontRight, vFrontCenter, vFrontRight
 
    'compute vfrontup
    D3DXVec3Scale vFrontUp, vUp, dY
    D3DXVec3Add vFrontUp, vFrontCenter, vFrontUp
    
    
    'front plane
    D3DXPlaneFromPointNormal FrustumPlanes(0), veye, vDir
    
    'back plane
    Dim vnegdir As D3DVECTOR
    D3DXVec3Scale vnegdir, vDir, -1
    D3DXPlaneFromPointNormal FrustumPlanes(1), vBackCenter, vnegdir
    
    'right plane
    D3DXPlaneFromPoints FrustumPlanes(2), veye, vBackRightTop, vBackRightBot
    
    'left plane
    D3DXPlaneFromPoints FrustumPlanes(3), veye, vBackLeftTop, vBackLeftBot
    
    'top plane
    D3DXPlaneFromPoints FrustumPlanes(4), veye, vBackLeftTop, vBackRightTop
    
    'bot plane
    D3DXPlaneFromPoints FrustumPlanes(5), veye, vBackRightBot, vBackLeftBot
    
End Sub

Private Function CheckSphere(Center As D3DVECTOR, Radius As Single) As Boolean
Dim TCenter As D3DVECTOR
Dim Matrix As D3DMATRIX, I As Long
Dim Dist As Single

D3DDevice.GetTransform D3DTS_WORLD, Matrix

D3DXVec3TransformCoord TCenter, Center, Matrix


For I = 0 To 5
    Dist = D3DXPlaneDotCoord(FrustumPlanes(I), TCenter)
        If Dist < -Radius Then
            CheckSphere = False 'not visible
        Exit Function
    End If
Next I
CheckSphere = True 'visible

End Function

'//A new wrapper function - so we can create vectors inline with function calls.
Private Function MakeVector(X As Single, Y As Single, Z As Single) As D3DVECTOR
    MakeVector.X = X
    MakeVector.Y = Y
    MakeVector.Z = Z
End Function



